import xarray as xr
import numpy as np
    

def common_preprocess(ds):
    try:
        ds = ds.drop('plev')
    except:
        pass
    try:
        ds = ds.drop('time_bnds')
    except:
        pass

    if 'prlr' in ds:
        ds = ds.rename({'prlr': 'pr'})

    unit_correction = {
        "tas": ("degC", 273.15),
        "tasmin": ("degC", 273.15),
        "tasmax": ("degC", 273.15),
        "pr": ("kg m-2 s-1", 0)
    }

    for key, (unit, offset) in unit_correction.items():
        if key in ds:
            if ds[key].attrs.get('units') != unit:
                ds[key] = ds[key] - offset if offset else ds[key]
                ds[key].attrs['units'] = unit

    ds.lat.attrs['units'] = 'degree'
    ds.lon.attrs['units'] = 'degree'

    return ds

def to_monthly(ds: xr.Dataset):

    # assign new coords in-place
    ds.coords["year"] = ("time", ds.time.dt.year.data)
    ds.coords["month"] = ("time", ds.time.dt.month.data)

    # reshape the array to (..., "month", "year") in-place
    return ds.set_index(time=("year", "month")).unstack("time", fill_value=np.nan)

def fix_coords(dataset):
    """ Function that sets longitudes in the -180,180 range and sorts
        latitudes.

        dataset: xarray object with 2D coords dims (lat,lon) or (latitude,longitude)
        return: arranged dataset
    """
    # Changes the name of the lon lat coords if necessary
    try:
        dataset = dataset.rename({'longitude':'lon','latitude':'lat'})
    except:
        #log = "no need to rename lon lat dims"
        pass
    try:
        dataset = dataset.rename({'X':'lon','Y':'lat'})
    except:
        #log = "no need to rename lon lat dims"
        pass

    # Converts longitudes to 0 360 domain
    dataset = dataset.assign_coords(lon=(((dataset.lon + 180) % 360) - 180))
    dataset = dataset.sortby('lon')
    dataset = dataset.sortby('lat')

    #dataset = dataset.assign_coords(month=("time", dataset.time.dt.month.values), 
    #                                year=("time", dataset.time.dt.year.values))

    # TODO: only works from xarray version 2022.09 onwards
    #dataset = dataset.set_xindex("month")
    #dataset = dataset.set_xindex("year")
    # Round times to start of the day
    try:
        dataset = dataset.assign_coords(time=dataset.time.dt.floor("D"))
    except:
        #log = "no need to reset hour un leadtimes"
        pass
    return dataset