

import xarray as xr
import pandas as pd

def _monthly2seasonal(
    ds: xr.Dataset,
    nmonths: int = 3,
    ) -> xr.Dataset:

    # creates a Jan to Dec monthly time series to allow seasonal means across years
    og_time_index = ds.time.to_index()
    new_time_index = pd.date_range(f"{og_time_index[0].year}-01-01",
                                f"{og_time_index[-1].year}-12-31", 
                                freq="1MS")

    ds = ds.reindex({"time": new_time_index})
    
    # compute seasonal means
    ds = ds.rolling(time=nmonths).mean()
    
    # reindex and drop NAs 
    ds = ds.reindex({"time": og_time_index})
    
    ds = ds.dropna(dim="time")
        
    return ds